@@ -47,7 +47,7 @@ class NextFieldGetter(Generic[FieldValueT]):
     def get_next_field(self, user: tg.User, value: Optional[FieldValueT]) -> Optional["FormField"]:
         if self.fields_by_name is None:
             raise RuntimeError(
-                "Next field getter hasn't been properly initialized, did you forget to call bind_form_fields?"
+                "Next field getter hasn't been properly initialized, did you forget to wrap your fields in a Form object?"
             )
         next_field_name = self.next_field_name_getter(user, value)
         if next_field_name is None:
@@ -399,8 +399,7 @@ class SingleSelectField(_EnumDefinedFieldMixin, FormField[Enum]):
         self, language: MaybeLanguage, current_value: Optional[FieldValueT] = None
     ) -> tg.ReplyKeyboardMarkup:
         kbd = tg.ReplyKeyboardMarkup(resize_keyboard=True, one_time_keyboard=True, row_width=self.menu_row_width)
-        for option in self.EnumClass:
-            kbd.add(tg.KeyboardButton(any_text_to_str(option.value, language)))
+        kbd.add(*[tg.KeyboardButton(any_text_to_str(option.value, language)) for option in self.EnumClass])
         return kbd
 
     def parse(self, message: tg.Message) -> Enum:
